
# operazioni da fare:
# 1) parsing dei messaggi nel file midi
# 2) eliminazione delle parole chiave e sostituzione con byte

import sys
import mido
import matplotlib.pyplot as plt
import numpy as np


mymidi =mido.MidiFile(sys.argv[1])
print("# Parsing file: "+sys.argv[1])

t0 = mymidi.tracks[0]
t1 = mymidi.tracks[1]
t2 = mymidi.tracks[2]

nmesg = len(t2)
print('Track 2 has '+str(nmesg)+' messages\n')
at = np.ndarray(shape=(127,nmesg), dtype=float)
x_at = np.ndarray(shape=(127,nmesg), dtype=float)
index = np.zeros(shape=(127,1), dtype=np.int32)
notes = np.zeros(shape=(1,1), dtype=np.int32)

note_on = np.zeros(shape=(1,1), dtype=np.int32)
time_on = np.zeros(shape=(1,1), dtype=np.float)
note_off = np.zeros(shape=(1,1), dtype=np.int32)
time_off = np.zeros(shape=(1,1), dtype=np.float)

fout = open(sys.argv[1]+'.parsed','w')
txb = mymidi.ticks_per_beat
tick_x_second = 98.372*txb/60.
for i, track in enumerate(mymidi.tracks):
    print('# Track {}: {}'.format(i, track.name))
    time = 0
    for msg in track:
        bb = msg.bytes()
        time += msg.time
        seconds = float(time/tick_x_second)
        if bb[0]==160:  # poly AT events
            #print( bb[0], bb[1], bb[2],seconds)
            fout.write(str(bb[0])+','+ str(bb[1])+','+ str(bb[2])+','+str(seconds)+'\n')
            idx_row =  int(bb[1])
            idx_col = int(index[idx_row])
            at[idx_row][idx_col] = float(bb[2])
            x_at[idx_row][idx_col] = float(seconds)
            index[idx_row] += 1
            #print('note, index, val: '+ str(idx_row)+', '+str(idx_col)+', '+str(at[idx_row][idx_col] ))
        if bb[0]==144:  # note ON messages
            #print( bb[0], bb[1], bb[2],time/tick_x_second)
            fout.write(str(bb[0])+','+ str(bb[1])+','+ str(bb[2])+','+str(time/tick_x_second)+'\n')
            note_on = np.append(note_on, int(bb[1]))
            time_on = np.append(time_on, seconds)
            if not int(bb[1]) in notes:
                notes = np.append(notes, int(bb[1]))
        if bb[0]==128:
            #print( bb[0], bb[1], bb[2],time/tick_x_second)
            fout.write(str(bb[0])+','+ str(bb[1])+','+ str(bb[2])+','+str(time/tick_x_second)+'\n')
            note_off = np.append(note_off, int(bb[1]))
            time_off = np.append(time_off, seconds)

fout.close()

print(notes)

#plt.style.use('_mpl-gallery')
## size and color:
#sizes = np.random.uniform(15, 80, len(x))
#colors = np.random.uniform(15, 80, len(x))
## plot
fig, ax = plt.subplots()

#ax.scatter(x_at, at, s=sizes, c=colors, vmin=0, vmax=130)

#ax.set(xlim=(0, 8), xticks=np.arange(1, 8),
       #ylim=(0, 500), yticks=np.arange(1, 8))
# naming the x axis
plt.xlabel('time (sec)')
# naming the y axis
plt.ylabel('poly-AT and notes')
# giving a title to my graph
plt.title('Note and poly-AT events (0-127)')
  
# show a legend on the plot
#plt.legend()

mycol = ['#ff0000', '#ff8000', '#ffb000', '#80ff00', '#00ff00', '#00ff80', '#00ffff', '#0080ff', '#0000ff', '#8000ff', '#ff00ff', '#ff0080']

plt.gca().set_prop_cycle(None)


for nota in notes:
    if nota != 0:
       index_color = nota % 12
       draw_color = mycol[index_color]
       y =  np.extract(x_at[int(nota)], at[int(nota)])
       x =  np.extract(x_at[int(nota)], x_at[int(nota)])
       plt.plot(x,y, label = "note="+str(nota), color=draw_color)

plt.gca().set_prop_cycle(None)

print(note_on)
note_on = np.delete(note_on, 0)
print(note_on)
print(note_off)
note_off = np.delete(note_off, 0)
print(note_off)
time_on = np.delete(time_on, 0)
time_off = np.delete(time_off, 0)

i=0
j=0
while i < len(note_on):
    while j < len(note_off):
        if note_off[j]==note_on[i]:
            print('x1, y1, x2, y2: '+str(time_on[i])+', '+str(note_on[i])+', '+str(time_off[j])+', '+str(note_off[j]))
            x1, y1 = [time_on[i], time_off[j]], [note_on[i], note_off[j]]            
            index_color = int(note_on[i]) % 12
            draw_color = mycol[index_color]
            plt.plot(x1, y1, marker='', color=draw_color)
            note_on = np.delete(note_on, i)
            time_on = np.delete(time_on, i)
            note_off = np.delete(note_off, j)
            time_off = np.delete(time_off, j)
            j=0
            i-=1
            break
        j+=1    
    i+=1
 

# plt.gca().set_prop_cycle(None)

# for i in range(len(note_off)):
    # plt.plot(time_off[i], note_off[i], marker='x', markersize=3)

plt.show()

